/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.axis.example;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;
import org.apache.axis.configuration.SimpleProvider;
import org.apache.axis.utils.Options;
import org.apache.axis.SimpleTargetedChain;
import org.apache.axis.transport.http.HTTPSender;

import org.globus.axis.transport.GSIHTTPSender;
import org.globus.axis.transport.GSIHTTPTransport;
import org.globus.axis.util.Util;
import org.globus.gsi.gssapi.auth.SelfAuthorization;

import java.net.URL;
import javax.xml.namespace.QName;
import javax.xml.rpc.ParameterMode;

public class Client
{
    public static void main(String [] args)
    {

	Util.registerTransport();

        try {
            Options options = new Options(args);
            
            String endpointURL = options.getURL();
            String textToSend;
            
            args = options.getRemainingArgs();
            if ((args == null) || (args.length < 1)) {
                textToSend = "<nothing>";
            } else {
                textToSend = args[0];
            }

	    SimpleProvider provider = new SimpleProvider();
	    
	    SimpleTargetedChain c = null;

	    c = new SimpleTargetedChain(new GSIHTTPSender());
	    provider.deployTransport("httpg", c);

	    c = new SimpleTargetedChain(new HTTPSender());
	    provider.deployTransport("http", c);
	    
            Service  service = new Service(provider);
            Call     call    = (Call) service.createCall();

	    // set globus credentials
	    /*
	    call.setProperty(GSIHTTPTransport.GSI_CREDENTIALS,
	                     cred);
	    */

	    // sets authorization type
	    call.setProperty(GSIHTTPTransport.GSI_AUTHORIZATION,
			     SelfAuthorization.getInstance());

	    // sets gsi mode
	    call.setProperty(GSIHTTPTransport.GSI_MODE,
			     GSIHTTPTransport.GSI_MODE_LIMITED_DELEG);

            call.setTargetEndpointAddress( new URL(endpointURL) );


	    call.setOperationName(new QName("MyService", 
					    "serviceMethod"));
	    
            call.addParameter( "arg1", 
			       XMLType.XSD_STRING, 
			       ParameterMode.IN);

	    call.setReturnType( XMLType.XSD_STRING );
	    
            String ret = (String) call.invoke( new Object[] { textToSend } );
            
            System.out.println("Service response : " + ret);
        } catch (Exception e) {
	    e.printStackTrace();
	}
    }
}
