/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.axis.handler;

import javax.servlet.http.HttpServletRequest;
import javax.security.auth.Subject;

import org.apache.axis.AxisFault;
import org.apache.axis.MessageContext;
import org.apache.axis.handlers.BasicHandler;
import org.apache.axis.transport.http.HTTPConstants;

import org.globus.axis.gsi.GSIConstants;
import org.globus.gsi.jaas.GlobusPrincipal;
import org.globus.gsi.jaas.UserNamePrincipal;

import org.apache.log4j.Logger;

public class CredentialHandler extends BasicHandler {

    private static Logger log =
        Logger.getLogger(CredentialHandler.class.getName());

    public static final String CALLER_SUBJECT =
	"callerSubject";
    
    public void invoke(MessageContext msgContext) throws AxisFault {
        log.debug("Enter: invoke");

	Object tmp = msgContext.getProperty(HTTPConstants.MC_HTTP_SERVLETREQUEST);

	if ((tmp == null) || !(tmp instanceof HttpServletRequest)) {
	    return;
	}

	HttpServletRequest req = (HttpServletRequest)tmp;
	
	// if httpg is access protocol in servlet engine, axis
	// will not set the TRANS_URL property correctly.
	// this is a workaround for that problem
	String url = req.getRequestURL().toString();
	tmp = msgContext.getProperty(MessageContext.TRANS_URL);
	if (tmp == null && url != null) {
	    msgContext.setProperty(MessageContext.TRANS_URL, url);
	}

	tmp = req.getAttribute(GSIConstants.GSI_CREDENTIALS);

	if (tmp != null) {
	    log.debug("Delegation performed. Setting credentials property.");
	    msgContext.setProperty(GSIConstants.GSI_CREDENTIALS, tmp);
	    Subject subject = getSubject(msgContext);
	    subject.getPrivateCredentials().add(tmp);
	} else {
	    log.debug("Delegation not performed. Not setting credentials property.");
	}
	

	tmp = req.getAttribute(GSIConstants.GSI_USER_DN);
	if (tmp != null) {
	    msgContext.setProperty(GSIConstants.GSI_USER_DN, tmp);
	    Subject subject = getSubject(msgContext);
	    subject.getPrincipals().add(new GlobusPrincipal((String)tmp));
	}

	tmp = req.getAttribute(GSIConstants.GSI_AUTH_USERNAME);
	if (tmp != null) {
	    msgContext.setProperty(GSIConstants.GSI_AUTH_USERNAME, tmp);
	    Subject subject = getSubject(msgContext);
	    subject.getPrincipals().add(new UserNamePrincipal((String)tmp));
	}
	
        log.debug("Exit: invoke");
    }

    protected Subject getSubject(MessageContext msgCtx) {
	Subject subject = 
	    (Subject)msgCtx.getProperty(CALLER_SUBJECT);
	if (subject == null) {
	    subject = new Subject();
	    msgCtx.setProperty(CALLER_SUBJECT, subject);
	}
	return subject;
    }

}
