/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ftp.dc;

import org.globus.ftp.HostPort;
import org.globus.ftp.vanilla.FTPServerFacade; 
import org.globus.ftp.vanilla.BasicServerControlChannel;

import java.net.Socket;

import org.apache.log4j.Logger;


/**
   This task wil initialize SocketBox's socket
   by connecting to the remote server.
   Any resulting exceptions are piped to the local control channel.
**/
public class ActiveConnectTask extends Task {
    static Logger logger = Logger.getLogger(ActiveConnectTask.class.getName());
    protected HostPort hostPort;
    protected BasicServerControlChannel control;
    protected SocketBox box;
    
    public ActiveConnectTask(HostPort hostPort, 
			     BasicServerControlChannel control,
			     SocketBox box) {
	if (box == null) {
	    throw new IllegalArgumentException(
					       "Socket box is null");
	}
	this.hostPort = hostPort;
	this.control = control;
	this.box = box;
    }
    
    public void execute() {
	try {
	    Socket mySocket = null;
	    try {
		
		// Facade's socket that we will attempt to initialize
		if (box.getSocket() != null) {
		    throw new IllegalArgumentException(
		            "socket should be null");
			    }
		
		logger.debug("connecting new socket to " + hostPort.toFtpCmdArgument());

		mySocket = new Socket(hostPort.getHost(), 
				      hostPort.getPort());
		
		// setting the Facade's socket
		box.setSocket(mySocket);
	    } catch (Exception e) {
		FTPServerFacade.exceptionToControlChannel(e, 
		        "active connection to server failed",
			 control);

		if (mySocket != null) {
		    mySocket.close();
		}
	    }
	} catch (Exception e) {
	    FTPServerFacade.cannotPropagateError(e);
	}

    }
} // class ActiveConnectTask


