/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ftp.dc;

import org.globus.ftp.vanilla.BasicServerControlChannel;
import org.globus.ftp.vanilla.FTPServerFacade;
import org.globus.ftp.Session;
import org.globus.ftp.DataSource;
import org.globus.ftp.DataSink;
import org.globus.ftp.HostPort;

import org.apache.log4j.Logger;

/**
   This task will start the transfer on the supplied socket.
   The socket is assumed to have been already connected to the
   remote server (for instance, by active connect task).
   It is little tricky: it will cause data channel to start
   a new thread. By the time this task completes, the new
   thread is running the transfer.
   Any resulting exceptions are piped to the local control channel.
**/
public class ActiveStartTransferTask extends Task {
	static Logger logger =
		Logger.getLogger(ActiveStartTransferTask.class.getName());
	HostPort hostPort;
	BasicServerControlChannel control;

	protected static final int STOR = 1, RETR = 2;
	int operation;
	DataSink sink;
	DataSource source;
	SocketBox box;
	Session session;
	DataChannelFactory factory;
	TransferContext context;

	public ActiveStartTransferTask(
		DataSink sink,
		BasicServerControlChannel control,
		SocketBox box,
		Session session,
		DataChannelFactory factory,
		TransferContext context) {
		this.sink = sink;
		init(STOR, control, box, session, factory, context);
	}

	public ActiveStartTransferTask(
		DataSource source,
		BasicServerControlChannel control,
		SocketBox box,
		Session session,
		DataChannelFactory factory,
		TransferContext context) {
		this.source = source;
		init(RETR, control, box, session, factory, context);
	}

	private void init(
		int operation,
		BasicServerControlChannel control,
		SocketBox box,
		Session session,
		DataChannelFactory factory,
		TransferContext context) {

		if (box == null) {
			throw new IllegalArgumentException("Socket box is null");
		}
		if (control == null) {
			throw new IllegalArgumentException("Control channel is null");
		}
		this.factory = factory;
		this.session = session;
		this.operation = operation;
		this.control = control;
		this.box = box;
		this.context = context;
	}

	public void execute() {
		try {
			try {
				if (box.getSocket() == null) {
					throw new IllegalArgumentException("socket is null");
				}
				logger.debug("active start transfer task executing");

				DataChannel dChannel = factory.getDataChannel(session, box);
				if (operation == STOR) {
					dChannel.startTransfer(sink, control, context);
				} else {

					dChannel.startTransfer(source, control, context);

				}
 			} catch (Exception e) {
	 			FTPServerFacade.exceptionToControlChannel(
		 			e,
					"startTransfer() failed",
					control);
			}
		} catch (Exception e) {
		    FTPServerFacade.cannotPropagateError(e);
		}

	}
}
