/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ftp.dc;

import java.io.InputStream;
import java.io.IOException;
import java.io.DataInputStream;

import org.globus.ftp.Buffer;

import org.apache.log4j.Logger;

public class EBlockImageDCReader
	extends EBlockAware
	implements DataChannelReader {

	boolean eodReceived = false;
	boolean willCloseReceived = false;

	private static Logger logger =
		Logger.getLogger(EBlockImageDCReader.class.getName());

	protected DataInputStream input;

	public void setDataStream(InputStream in) {
		input = new DataInputStream(in);
	}

	/** @return true if at least once received
	the "server will close the connection" signal
	 */
	public boolean willCloseReceived() {
		return willCloseReceived;
	}

	public Buffer read() throws IOException {

		//EOD received in previous read
	    if (eodReceived) {
		return null;
	    }
	    // WILL_CLOSE received in previous read
	    if (willCloseReceived) {
		return null;
	    }

		byte desc = input.readByte();
		long size = input.readLong();
		long offset = input.readLong();

		boolean eof = (desc & EOF) != 0;
		boolean eod = (desc & EOD) != 0;

		// if closing flag not yet received,
		// check this buffer for closing flag
		willCloseReceived = (desc & WILL_CLOSE) != 0;
		if (willCloseReceived) {
		    logger.info("Received the CLOSE flag");
		}

		if (logger.isDebugEnabled()) {
			logger.debug(desc + " " + size + " " + offset);
			logger.debug(eof + " " + eod);
		}

		if (eod) {
			this.eodReceived = true;
			context.eodTransferred();
			logger.info(
				"received EOD. Still expecting: "
					+ ((context.getEodsTotal() == context.UNDEFINED)
						? "?"
						: Integer.toString(
							context.eodsTotal - context.eodsTransferred)));
		}

		if (eof) {
			context.setEodsTotal((int) offset);
			logger.info(
				"Received EOF. expecting total EODs: "
					+ context.getEodsTotal());
			return null;

		} else {
			byte[] bt = new byte[(int) size];
			input.readFully(bt);
			return new Buffer(bt, (int) size, offset);
		}
	}

	public void close() throws IOException {
		// we want to reuse the socket
		input.close();
	}
}
