/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ftp.dc;

import org.globus.ftp.GridFTPSession;
import org.globus.ftp.DataChannelAuthentication;
import org.globus.ftp.HostPort;
import org.globus.ftp.vanilla.FTPServerFacade;
import org.globus.ftp.vanilla.BasicServerControlChannel;
import org.globus.ftp.extended.GridFTPServerFacade;

import java.net.Socket;

import org.apache.log4j.Logger;

/**
   Unlike in the parent class, here we use authentication
   and protection.
 **/
public class GridFTPActiveConnectTask extends ActiveConnectTask {

	private static Logger logger =
		Logger.getLogger(GridFTPActiveConnectTask.class.getName());

	protected GridFTPSession gSession;

	public GridFTPActiveConnectTask(
		HostPort hostPort,
		BasicServerControlChannel control,
		SocketBox box,
		GridFTPSession gSession) {
		super(hostPort, control, box);
		this.gSession = gSession;
	}

	public void execute() {
		Socket mySocket = null;
		try {
			logger.debug(
				"connecting new socket to " + hostPort.toFtpCmdArgument());

			mySocket = new Socket(hostPort.getHost(), hostPort.getPort());

			// set TCP buffer size

			if (gSession.TCPBufferSize != gSession.SERVER_DEFAULT) {
			    logger.debug("setting socket's TCP buffer size to " 
					 + gSession.TCPBufferSize);
			    mySocket.setReceiveBufferSize(gSession.TCPBufferSize);
			    mySocket.setSendBufferSize(gSession.TCPBufferSize);
			}

			if (!gSession
				.dataChannelAuthentication
				.equals(DataChannelAuthentication.NONE)) {
					
				logger.debug("authenticating");
				mySocket = GridFTPServerFacade.authenticate(mySocket, true,
									    // this IS client socket
									    gSession.credential,
									    gSession.dataChannelProtection,
									    gSession.dataChannelAuthentication);
		
			} else {
				logger.debug("not authenticating");
			}
			
			// setting the Facade's socket list

			// synchronize to prevent race condidion against
			// the section in GridFTPServerFacade.setTCPBufferSize
			synchronized (box) {
			    box.setSocket(mySocket);
			}

		} catch (Exception e) {
			logger.debug("intercepted: " + e.toString());
			FTPServerFacade.exceptionToControlChannel(
				e,
				"active connection to server failed",
				control);
			try {
				if (mySocket != null) {
					mySocket.close();
				}
			} catch (Exception second) {
			};
		}
	}
} // class
