package org.globus.ftp.dc;

import java.net.Socket;

import org.apache.log4j.Logger;

/**
   A container for Socket, making it available to pass a null socket reference.

   We use asynchronously active connect task to initialize socket,
   and active start transfer task to run it.
   We need to pass the socket reference, which is sometimes null
   (before initialization).
   This is a sane way to do it; a simple socket container.

   Additionally, the box contains a flag that states whether the socket 
   is currently in use, ie whether it is assigned to some data channel.
   It is needed in GridFTP for data channel reuse.
 **/

public class ManagedSocketBox implements SocketBox {

    public static final int FREE = 1;
    public static final int BUSY = 2;
    
    public static final boolean REUSABLE = true;
    public static final boolean NON_REUSABLE = false;

	public static Logger logger = Logger.getLogger(ManagedSocketBox.class.getName());
	
	protected Socket socket;

	protected int status = FREE;
	
	// the socket pool in which this socket belongs
	protected SocketPool myPool;

	// should the socket be reused? by default, yes
	protected boolean reusable = true;

	// index in the socketPool
	protected int index = -1;

	public ManagedSocketBox() {
	}

	public void setSocket(Socket newSocket) {
		logger.debug(
			newSocket == null ? "Setting socket to null" : "Setting socket");
		socket = newSocket;
	}

	public Socket getSocket() {
		return socket;
	}
    
	public void setStatus(int s) {
		this.status = s;
	}

	public int getStatus() {
		return status;
	}
	
	public void setReusable(boolean r) {
		this.reusable = r;
	}

	public boolean isReusable() {
		return reusable;
	}	

}
