/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ftp.test;

import org.globus.ftp.Session;
import org.globus.ftp.FTPClient;
import org.globus.ftp.DataSink;
import org.globus.ftp.HostPort;
import org.globus.ftp.Buffer;
import org.globus.ftp.FileInfo;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.log4j.Logger;

import java.io.IOException;
import java.util.Vector;
import java.io.ByteArrayOutputStream;

public class FTPClientListTest extends TestCase {

    private static Logger logger = 
	Logger.getLogger(FTPClientListTest.class.getName());

    public FTPClientListTest(String name) {
	super(name);
    }

    public static void main (String[] args) throws Exception{
	junit.textui.TestRunner.run(suite());	
    }

    public static Test suite ( ) {
	return new TestSuite(FTPClientListTest.class);
    }

    public void test1() throws Exception {
	logger.info("show list output");

	FTPClient src = new FTPClient(TestEnv.serverFHost, TestEnv.serverFPort);
	src.authorize(TestEnv.serverFUser, TestEnv.serverFPassword);

	src.setType(Session.TYPE_ASCII);
	src.changeDir(TestEnv.serverFDir);
	Vector v = src.list();
	logger.debug("list received");
	while (! v.isEmpty()) {
	    FileInfo f = (FileInfo)v.remove(0); 
	    logger.info(f.toString());
	}

	src.close();
    }
    
    
    public void test2() throws Exception {
	logger.info("test two consective list, using both list functions");

	FTPClient src = new FTPClient(TestEnv.serverFHost, TestEnv.serverFPort);
	src.authorize(TestEnv.serverFUser, TestEnv.serverFPassword);

	String output1 = null;
	String output2 = null;

	// using list()

	src.changeDir(TestEnv.serverFDir);
	Vector v = src.list();
	logger.debug("list received");
	StringBuffer output1Buffer = new StringBuffer();
	while (! v.isEmpty()) {
	    FileInfo f = (FileInfo)v.remove(0); 
	    output1Buffer.append(f.toString()).append("\n");

	}
	output1 = output1Buffer.toString();



	// using list(String,String, DataSink)


	HostPort hp2 = src.setPassive();
	src.setLocalActive();

	final ByteArrayOutputStream received2= new ByteArrayOutputStream(1000);

	// unnamed DataSink subclass will write data channel content
	// to "received" stream.

	 src.list("*", "-d", new DataSink(){
		public void write(Buffer buffer) 
		    throws IOException{
		    logger.debug("received " + buffer.getLength() +
				 " bytes of directory listing");
		    received2.write(buffer.getBuffer(),
				   0,
				   buffer.getLength());
		}
		public void close() 
		    throws IOException{};
	     });

	// transfer done. Data is in received2 stream.

	 output2 = received2.toString();
	 logger.debug(output2);

	 src.close();
    }    
}
