/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.ftp.test;

import org.globus.ftp.HostPort;
import org.globus.ftp.HostPortList;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.log4j.Logger;

/**
   Test HostPortList
 **/
public class HostPortListTest extends TestCase {

    private static Logger logger = 
	Logger.getLogger(HostPortListTest.class.getName());

    private static String nl = System.getProperty("line.separator");
    private static String space = " ";

    String param1 = "140,221,65,198,173,202";
    String msg1 ="Entering Striped Passive Mode" + nl +
	space + param1 + nl +
	"229 End";

    String hp1str = "140,221,65,198,173,202";
    String hp2str = "140,221,65,198,1,50";
    String param2 = hp1str + space + hp2str;
    String msg2 ="Entering Striped Passive Mode" + nl +
	space + hp1str + nl +
	space + hp2str + nl +
	"229 End";

    String msg3 = "MODE E ok.";

    String msg4 = "Extensions supported:" + nl +
	space + "REST STREAM" + nl + 
	space + "ESTO" + nl + 
	space + "ERET" + nl + 
	space + "MDTM" + nl + 
	space + "SIZE" + nl + 
	space + "PARALLEL" + nl + 
	space + "DCAU" + nl + 
	"211 END";

    String hp5str = "127,0,0,1,100,0";
    
    String param5 = hp1str + space + hp2str + space + hp5str;

    String msg5 ="Entering Striped Passive Mode" + nl +
	space + hp1str + nl +
	space + hp2str + nl +
	space + hp5str + nl +
	"229 End";


    public static void main(String[] argv) {
	junit.textui.TestRunner.run (suite());
    }
    
    public static Test suite() {
	return new TestSuite(HostPortListTest.class);
    }

    public HostPortListTest(String name) {
	super(name);
    }

    public void testString() {
	logger.info("testing construction from String");
	testString(msg1, param1);
	testString(msg2, param2);
	testConstructorError(msg3);
	testConstructorError(msg4);

    }

    public void testHP() {
	logger.info("testing construction from HostPort object");
	HostPort hp1 = new HostPort(param1);
	HostPortList hpl1 = new HostPortList();
	hpl1.add(hp1);
	//double checking - make sure this does not change internal state
	testObject(hpl1, param1);
	testObject(hpl1, param1);

	HostPortList hpl2 = new HostPortList();
        hpl2.add(new HostPort(hp1str));
	hpl2.add(new HostPort(hp2str));
	testObject(hpl2, param2);
	testObject(hpl2, param2);
    }

    public void testMixed() {
	logger.info("testing construction from String, and later modification by add(HostPort)");

	HostPortList hpl2 = new HostPortList(msg1);
	hpl2.add(new HostPort(hp2str));
	testObject(hpl2, param2);
	testObject(hpl2, param2);
	hpl2.add(new HostPort(hp5str));
	testObject(hpl2, param5);
	testObject(hpl2, param5);
    }

    /**
       make sure that message "msg" is properly converted 
       to the SPOR command argument "arg"
    **/
    private void testString(String msg, String ftpCmdArg) {
	logger.info("expected ftp command argument: " + ftpCmdArg);
	String result = new HostPortList(msg).toFtpCmdArgument();
	logger.debug("result string: " + result);
	assertTrue( result.equals(ftpCmdArg));
	logger.debug("okay, created properly.");
    }

    private void testObject(HostPortList hpl, String ftpCmdArg) {
	logger.info("expected ftp command argument: " + ftpCmdArg);
	String msg = hpl.toFtpCmdArgument(); 
	logger.info("result string: " + msg);
	assertTrue( msg.equals(ftpCmdArg));
	logger.debug("okay, created properly.");
    }
    /** 
	assume this is a bad argument to HostPortList constructor.
	make sure the constructor throws an exception.
    **/
    private void testConstructorError(String msg) {
	logger.info("checking bad message: " + msg);
	boolean threwOk = false;
	try {
	    new HostPortList(msg);
	} catch (IllegalArgumentException e) {
	    threwOk = true;
	} 
	
	if (! threwOk ) {
	    fail("HostPortList constructor did not throw an exception when it should have");
	}
	logger.debug("okay, throws exception as expected.");
    }

}

