/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.gram.internal;

import org.globus.util.http.HttpResponse;
import org.globus.util.log4j.CoGLevel;

import java.io.IOException;
import java.io.InputStream;

import org.apache.log4j.Logger;

public class GatekeeperReply extends HttpResponse {

    private static Logger logger =
	Logger.getLogger(GatekeeperReply.class.getName());

    public int protocolVersion   = -1;
    public int status            = -1;
    public String jobManagerUrl  = null;
    public int failureCode       = -1;
    public int jobFailureCode    = -1;

    public GatekeeperReply(InputStream in) throws IOException {
	super(in);    
	charsRead = 1;
	if (contentLength > 0) myparse();
    }

    protected void myparse() throws IOException {
	
	String line, tmp;
	
        while(charsRead < contentLength) {
            line = readLine(input);
	    if (line.length() == 0) break;

	    if (logger.isEnabledFor(CoGLevel.TRACE)) {
		logger.log(CoGLevel.TRACE, line);
	    }

            tmp = getRest(line.trim());   
	    
	    if (line.startsWith("protocol-version:")) {
	      protocolVersion = Integer.parseInt(tmp);
	    } else if (line.startsWith("status:")) {
		status = Integer.parseInt(tmp);
	    } else if (line.startsWith("job-manager-url:")) {
		jobManagerUrl = tmp;
	    } else if (line.startsWith("failure-code:")) {
		failureCode = Integer.parseInt(tmp);
	    } else if (line.startsWith("job-failure-code:")) {
		jobFailureCode = Integer.parseInt(tmp);
	    }
	}
    }

    public String toString() {
	StringBuffer buf = new StringBuffer();

	buf.append(super.toString());
	
	buf.append("Protocol-version : " + protocolVersion + "\n");
	buf.append("Status           : " + status);
	if (jobManagerUrl != null) {
	    buf.append("\nJob-manager-url  : " + jobManagerUrl);
	}
	if (failureCode >= 0) {
	    buf.append("\nFailure-code     : " + failureCode);
	}
	if (jobFailureCode >= 0) {
            buf.append("\nJob failure code     : " + jobFailureCode);
        }
	
	return buf.toString();
    }
}
	



