/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.gsi.gssapi.auth;

import org.ietf.jgss.GSSContext;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;
import org.ietf.jgss.GSSName;

import org.apache.log4j.Logger;

/**
 * Implements the simplest authorization mechanism that does
 * not do any authorization checks.
 */
public class NoAuthorization extends Authorization {

    private static Logger logger =
	Logger.getLogger(NoAuthorization.class.getName());

    private static NoAuthorization authorization;
    
    /**
     * Returns a single instance of this class.
     *
     * @return the instance of this class.
     */
    public synchronized static NoAuthorization getInstance() {
	if (authorization == null) {
	    authorization = new NoAuthorization();
	}
	return authorization;
    }
    
    /**
     * Always returns null.
     */
    public GSSName getExpectedName(GSSCredential cred, String host) 
	throws GSSException {
        return null;
    }

    /**
     * Performs no authorization checks. The function is always
     * successful. It does not throw any exceptions.
     *
     */
    public void authorize(GSSContext context, String host)
	throws AuthorizationException {
	logger.info("Authorization: NONE");
    }
    
}
