/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.gsi.gssapi.net.impl;

import java.io.InputStream;
import java.io.IOException;
import java.io.EOFException;

import org.globus.gsi.gssapi.net.GssSocket;
import org.globus.gsi.gssapi.net.GssInputStream;
import org.globus.gsi.gssapi.SSLUtil;

import org.ietf.jgss.GSSContext;

public class GSIGssInputStream extends GssInputStream {
  
    // 32Mb 
    private static final int MAX_LEN = 32 * 1024 * 1024;

    protected byte [] header;
    protected int mode;

    public GSIGssInputStream(InputStream in, GSSContext context) {
	super(in, context);
	this.header = new byte[4];
	this.mode = -1;
    }
  
    protected void readMsg()
	throws IOException {
	byte [] token = readToken();
	if (token == null) {
	    this.buff = null;
	} else {
	    this.buff = unwrap(token);
	}
	this.index = 0;
    }

    public int getWrapMode() {
	return this.mode;
    }

    public byte[] readHandshakeToken()
	throws IOException {
	byte [] token = readToken();
	if (token == null) {
	    throw new EOFException();
	}
	return token;
    }

    protected byte[] readToken()
	throws IOException {
	byte[] buf = null;
	if (SSLUtil.read(this.in, this.header, 0, this.header.length) < 0) {
	    return null;
	}
	if (SSLUtil.isSSLPacket(this.header)) {
	    this.mode = GssSocket.SSL_MODE;
	    int l2 = this.in.read();
	    if (l2 < 0) {
		throw new EOFException();
	    }
	    int len = SSLUtil.toShort(this.header[3], (byte)l2);
	    buf = new byte[5 + len];
	    System.arraycopy(this.header, 0, buf, 0, this.header.length);
	    buf[4] = (byte)l2;
	    if (SSLUtil.read(this.in, buf, 5, len) < 0) {
		return null;
	    }
	} else {
	    this.mode = GssSocket.GSI_MODE;
	    int len = SSLUtil.toInt(this.header, 0);
	    if (len > MAX_LEN) {
		throw new IOException("Token length " + len + " > " + MAX_LEN);
	    }
	    buf = new byte[len];
	    if (SSLUtil.read(this.in, buf, 0, buf.length) < 0) {
		return null;
	    }
	}
	return buf;
    }
    
}
