/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.io.streams;

import java.io.IOException;

import org.globus.ftp.GridFTPClient;
import org.globus.ftp.Session;
import org.globus.ftp.DataChannelAuthentication;
import org.globus.ftp.exception.FTPException;
import org.globus.gsi.gssapi.auth.Authorization;
import org.globus.gsi.gssapi.auth.HostAuthorization;

import org.ietf.jgss.GSSCredential;

public class GridFTPInputStream extends FTPInputStream {

    public GridFTPInputStream(GSSCredential cred,
			      String host, 
			      int port, 
			      String file) 
	throws IOException, FTPException {
	this(cred, HostAuthorization.getInstance(),
	     host, port, 
	     file, true, Session.TYPE_IMAGE, true);
    }

    public GridFTPInputStream(GSSCredential cred,
			      Authorization auth,
			      String host, 
			      int port, 
			      String file,
			      boolean reqDCAU) 
	throws IOException, FTPException {
	this(cred, auth, host, port, 
	     file, true, Session.TYPE_IMAGE, reqDCAU);
    }

    public GridFTPInputStream(GSSCredential cred,
			      Authorization auth,
			      String host, 
			      int port, 
			      String file,
			      boolean passive,
			      int type,
			      boolean reqDCAU) 
	throws IOException, FTPException {
	GridFTPClient gridFtp = new GridFTPClient(host, port);
	gridFtp.setAuthorization(auth);
	gridFtp.authenticate(cred);
	
	if (gridFtp.isFeatureSupported("DCAU")) {
	    if (!reqDCAU) {
		gridFtp.setDataChannelAuthentication(DataChannelAuthentication.NONE);
	    }
	} else {
	    gridFtp.setLocalNoDataChannelAuthentication();
	}

	ftp = gridFtp;
	
	get(passive, type, file);
    }

}
