/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.net;

import java.net.URL;
import java.net.URLConnection;

import org.globus.gsi.GSIConstants;
import org.globus.gsi.gssapi.auth.Authorization;
import org.globus.gsi.gssapi.auth.GSSAuthorization;

import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSName;
import org.ietf.jgss.GSSException;

public abstract class GSIURLConnection extends URLConnection {

    protected GSSCredential credentials;
    protected Authorization authorization;
    protected int delegationType;

    /**
     * Subclasses must overwrite.
     */
    protected GSIURLConnection(URL url) {
	super(url);
	this.delegationType = GSIConstants.DELEGATION_NONE;
	this.authorization = null; // no authorization?
    }

    public abstract void disconnect();

    public void setCredentials(GSSCredential credentials) {
        this.credentials = credentials;
    }

    public GSSCredential getCredentials() {
        return credentials;
    }

    public void setAuthorization(Authorization auth) {
        authorization = auth;
    }

    public Authorization getAuthorization() {
        return authorization;
    }
    
    public void setDelegationType(int delegationType) {
	this.delegationType = delegationType;
    }

    public int getDelegationType() {
	return delegationType;
    }

    protected GSSName getExpectedName() throws GSSException {
        if (this.authorization instanceof GSSAuthorization) {
            GSSAuthorization auth = (GSSAuthorization)this.authorization;
            return auth.getExpectedName(this.credentials,
                                        this.url.getHost());
        } else {
            return null;
        }
    }

}
