/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.security.gridmap.tests;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;

import java.io.IOException;
import java.io.ByteArrayInputStream;

import org.globus.security.gridmap.GridMap;

public class GridMapTest extends TestCase {

    String s =
	"\"/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Katarzyna (Kate) Keahey\" keahey,kate\r\n" +
	"# this is a comment\r\n" +
	"           \r\n" + 
	"\"/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Michael Dvorak\" dvorak2\r\n" + 
	"\"/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Michael Dvorak\" dvorak";
    
    public GridMapTest(String name) {
	super(name);
    }
    
    public static void main (String[] args) {
	junit.textui.TestRunner.run (suite());
    }

    public static Test suite() {
	return new TestSuite(GridMapTest.class);
    }

    public void test1() {
	
	GridMap map = new GridMap();

	try {
	    map.load(new ByteArrayInputStream(s.getBytes()));   
	} catch (IOException e) {
	    fail(e.getMessage());
	    return;
	}

	assertEquals("keahey", 
		     map.getUserID("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Katarzyna (Kate) Keahey"));

	assertEquals("dvorak2", 
		     map.getUserID("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Michael Dvorak"));

	assertEquals(null, 
		     map.getUserID("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Jarek Gawor"));

	String [] rs = null;
	String [] expected = null;

	rs = map.getUserIDs("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Katarzyna (Kate) Keahey");
	expected = new String [] {"keahey", "kate"};

	assertEquals(expected.length, rs.length);
	for (int i=0;i<expected.length;i++) {
	    assertEquals(expected[i], rs[i]);
	}

	rs = map.getUserIDs("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Michael Dvorak");
	expected = new String [] {"dvorak2", "dvorak"};
	
        assertEquals(expected.length, rs.length);
        for (int i=0;i<expected.length;i++) {
            assertEquals(expected[i], rs[i]);
        }
	
    }
    
    public void test2() {
	GridMap map = new GridMap();

	String username = System.getProperty("user.name");

	assertEquals(username, map.getUserID("whatever") );

	System.setProperty("user.name", "ROot");

	assertEquals(null, map.getUserID("whatever") );

	System.setProperty("user.name", "AdminISTRATOR");

	assertEquals(null, map.getUserID("whatever") );

	System.setProperty("user.name", username);

	assertEquals(username, map.getUserID("whatever2") );
    }

    public void test3() {
	
	GridMap map = new GridMap();

	try {
	    map.load(new ByteArrayInputStream(s.getBytes()));   
	} catch (IOException e) {
	    fail(e.getMessage());
	    return;
	}

	assertEquals(true,
		     map.checkUser("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Katarzyna (Kate) Keahey",
				   "keahey"));

	assertEquals(true,
		     map.checkUser("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Katarzyna (Kate) Keahey",
				   "kate"));

	assertEquals(true,
		     map.checkUser("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Michael Dvorak",
				   "dvorak"));

	assertEquals(false,
		     map.checkUser("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Michael Dvorak",
				   "dvorakkkkk"));

	assertEquals(false,
		     map.checkUser("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Michael Dvorakkkk",
				   "dvorak"));
	
	assertEquals(false, 
		     map.checkUser("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Jarek Gawor", 
				   "gawor"));
    }    

    public void test4() {
	
	GridMap map = new GridMap();

	try {
	    map.load(new ByteArrayInputStream(s.getBytes()));   
	} catch (IOException e) {
	    fail(e.getMessage());
	    return;
	}
	
	assertEquals("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Michael Dvorak",
		     map.getGlobusID("dvorak"));

	assertEquals("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Katarzyna (Kate) Keahey",
		     map.getGlobusID("keahey"));

	assertEquals("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Katarzyna (Kate) Keahey",
		     map.getGlobusID("kate"));

	assertEquals(null, map.getGlobusID("gawor"));

    }

    public void test5() {
	  
	GridMap map = new GridMap();

	try {
	    map.load(new ByteArrayInputStream(s.getBytes()));   
	} catch (IOException e) {
	    fail(e.getMessage());
	    return;
	}

	map.map("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Jarek Gawor", 
		"gawor");

	assertEquals("gawor", 
		     map.getUserID("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Jarek Gawor"));

	map.map("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Michael Dvorak",
		"dvorak2");

	assertEquals(true,
		     map.checkUser("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Michael Dvorak",
				   "dvorak"));
	
	assertEquals(true,
		     map.checkUser("/O=Grid/O=Globus/OU=mcs.anl.gov/CN=Michael Dvorak",
				   "dvorak2"));
	
    }
}

