/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.tomcat.catalina.net;

import java.net.Socket;
import java.io.IOException;

import org.globus.security.gridmap.GridMap;
import org.globus.gsi.gssapi.auth.AuthorizationException;
import org.globus.gsi.gssapi.net.impl.GSIGssSocket;
import org.globus.common.ChainedIOException;

import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSContext;
import org.ietf.jgss.GSSException;

public class GSISocket extends GSIGssSocket {

    protected GridMap gridMap;

    protected GSSCredential delegatedCreds;
    protected String userID;
    protected String userDN;
    
    private boolean init = false;
    
    public GSISocket(Socket socket, GSSContext context) {
	super(socket, context);
	setAuthorization(null);
    }

    public void setGridMap(GridMap gridMap) {
	this.gridMap = gridMap;
    }

    public GSSCredential getDelegatedCredentials() {
	return delegatedCreds;
    }

    public String getAuthorizedUserName() {
	return userID;
    }
    
    public String getAuthorizedUserDN() {
	return userDN;
    }

    public synchronized void startHandshake()
	throws IOException {
	super.startHandshake();
	
	if (init) {
	    return;
	}

	GSSContext context = getContext();

	try {
	    delegatedCreds = context.getDelegCred();
	    userDN = context.getSrcName().toString();
	} catch (GSSException e) {
	    throw new ChainedIOException("Failed to retreive context properties", e);
	}
	
	userID = gridMap.getUserID(userDN);
	
	if (userID == null) {
	    // FIXME: should close socket?
	    throw new AuthorizationException("User not authenticated: " +
					     userDN);
	}
	
	init = true;
    }
    
}
