/*
This file is licensed under the terms of the Globus Toolkit Public
License, found at http://www.globus.org/toolkit/download/license.html.
*/
package org.globus.util.http;

import java.io.InputStream;
import java.io.IOException;

import org.globus.util.log4j.CoGLevel;

import org.apache.log4j.Logger;

public class HttpResponse {

    private static Logger logger =
	Logger.getLogger(HttpResponse.class.getName());

    protected InputStream input;
    protected long charsRead  = 0;

    public String httpType    = null;
    public String httpMsg     = null;
    public int httpCode       = -1;
    public long contentLength  = -1;
    public String connection  = null;
    public String contentType = null;
    public String server      = null;
    public boolean chunked    = false;
    public String location    = null;

    public HttpResponse(InputStream in) throws IOException {
	input = in;
	parse();
    }

    /**
     * Read a line of text from the given Stream and return it
     * as a String.  Assumes lines end in CRLF.
     */
    protected String readLine(InputStream in) throws IOException {
	StringBuffer buf = new StringBuffer();
	int c, length = 0;
	
	while(true) {
	    c = in.read();
	    
	    if (c == -1 || c == '\n' || length > 512) {
		charsRead++;
		break;
	    } else if (c == '\r') { 
		in.read(); 
		charsRead+=2;
		break;
	    } else {	    
		buf.append((char)c);
		length++;	
	    }
	}
	
	charsRead += length;
	return buf.toString();
    } 
    
    public static String getRest(String line) {
	int pos = line.indexOf(":");
	if (pos == -1) {
	    return null;
	} else
	    return line.substring(pos+1).trim();
    }
    
    public void parseHttp(String line) {
	
	int p1 = line.indexOf(" ");
	if (p1 == -1) {
	    return;
	}
	httpType = line.substring(0,p1);
	
	int p2 = line.indexOf(" ",p1+1);
	
	String tmp;
	
	if (p2 == -1) {
	    tmp = line.substring(p2);
	} else {
	    tmp = line.substring(p1,p2);
	    httpMsg = line.substring(p2).trim();
	}
	httpCode = Integer.parseInt(tmp.trim());
    }

    private void parse() throws IOException {
	String line, tmp;
	
	line = readLine(input);
	if (logger.isEnabledFor(CoGLevel.TRACE)) {
	    logger.log(CoGLevel.TRACE, line);
	}
	parseHttp(line);
	
	while ( (line=readLine(input)).length() != 0 ) {
	    if (logger.isEnabledFor(CoGLevel.TRACE)) {
		logger.log(CoGLevel.TRACE, line);
	    }
	    tmp = getRest(line);
	    
	    if (line.startsWith(HTTPProtocol.CONNECTION)) {
		connection = tmp;
	    } else if (line.startsWith(HTTPProtocol.SERVER)) {
		server = tmp;
	    } else if (line.startsWith(HTTPProtocol.CONTENT_TYPE)) {
		contentType = tmp;
	    } else if (line.startsWith(HTTPProtocol.CONTENT_LENGTH)) {
		contentLength = Long.parseLong(tmp.trim());
	    } else if (line.startsWith(HTTPProtocol.CHUNKED)) {
		chunked = true;
	    } else if (line.startsWith(HTTPProtocol.LOCATION)) {
		location = tmp;
	    }
	}
    }

    /** Generates a string representation of the http header
     * 
     * @return <code>String</code> a string representation of the http header
     */
    public String toString() {
	StringBuffer buf = new StringBuffer();
	
	buf.append("Http    : " + httpType + "\n");
	buf.append("Message : " + httpMsg + "\n");
	buf.append("Code    : " + httpCode + "\n");

	if (server != null) {
	    buf.append("Server  : " + server + "\n");
	}

	buf.append("Length  : " + contentLength + "\n");
	buf.append("Chunked : " + chunked + "\n");
	buf.append("Type    : " + contentType + "\n");
	
	if (connection != null) {
	    buf.append("Connection   : " + connection + "\n");
	}

	if (location != null) {
	    buf.append("Location     : " + location + "\n");
	}
	
	return buf.toString();
    }
    
}
